<?php
/**
 * TimeValidator ... .
 *
 * <br/><br/>
 *
 * 指定したパラメータは連想配列になっていて
 * Hour, Minuteの添え字があることを前提とする。Secondは任意とし、添え字にない場合は
 * デフォルト値params['def_second']を代入する。
 * デフォルトはは24時間表記チェック、params['time_kind']を12にすると12時間表記チェック
 *
 * <br/><br/>
 *
 * Initialization Parameters:
 *
 * <br/><br/>
 *
 * <table border="0" cellpadding="3" cellspacing="0">
 *     <tr>
 *         <th>Name</th>
 *         <th>Type</th>
 *         <th>Default</th>
 *         <th>Required</th>
 *         <th>Description</th>
 *     </tr>
 *     <tr>
 *         <td valign="top">max</td>
 *         <td valign="top">Array</td>
 *         <td valign="top">n/a</td>
 *         <td valign="top">no</td>
 *         <td valign="top">a maximum date</td>
 *     </tr>
 *     <tr>
 *         <td valign="top">min</td>
 *         <td valign="top">Array</td>
 *         <td valign="top">n/a</td>
 *         <td valign="top">no</td>
 *         <td valign="top">a minimum date</td>
 *     </tr>
 * </table>
 *
 * <br/><br/>
 *
 * Error Messages:
 *
 * <br/><br/>
 *
 * <table border="0" cellpadding="3" cellspacing="0">
 *     <tr>
 *         <th>Name</th>
 *         <th>Default</th>
 *     </tr>
 *     <tr>
 *         <td valign="top">date_error</td>
 *         <td valign="top">Value is invalid date</td>
 *     </tr>
 *     <tr>
 *         <td valign="top">max_error</td>
 *         <td valign="top">Value is too long</td>
 *     </tr>
 *     <tr>
 *         <td valign="top">min_error</td>
 *         <td valign="top">Value is too short</td>
 *     </tr>
 * </table>
 *
 * @author  Taro Aso
 * @package mojavi
 * @package validators
 * @since   1.0
 */
class TimeValidator extends Validator
{

    /**
     * Create a new DateValidator instance.
     *
     * @access public
     * @since  1.0
     */
    function TimeValidator ()
    {
        // 初期値の設定
        $this->params['max']         = '';
        $this->params['max_error']   = 'Value is too long';
        $this->params['min']         = '';
        $this->params['min_error']   = 'Value is too short';
        $this->params['time_error']  = 'Value is invalid date';
        $this->params['time_kind']   = 24;
        $this->params['def_second']  = "00";

    }

    /**
     * Execute this validator.
     *
     * @param string     A user submitted parameter value(prefix).
     * @param string     The error message variable to be set, if an error
     *                   occurs.
     * @param Controller A Controller instance.
     * @param Request    A Request instance.
     * @param User       A User instance.
     *
     * @return bool <b>TRUE</b>, if the validator completes successfully,
     *              otherwise <b>FALSE</b>.
     *
     * @access public
     * @since  1.0
     */
    function execute (&$value, &$error, &$controller, &$request, &$user)
    {
        // 時間データ取得
        $hour   = $value['Hour'];
        $minute = $value['Minute'];
        $second = $value['Second'];

        //[ 秒が空？ ]
        if ( strlen((string) $second) <= 0) {
            $second = $this->params['def_second'];
        }

        // 全て入力がない場合は正常（必須はValidatorManagerで見る）
        if( !isset($hour) && !isset($minute) && !isset($second) ){
            return TRUE;
        }
        // 歯抜けの場合は時間不正
        else if( !isset($hour) || !isset($minute) || !isset($second) ){
            $error = $this->params['time_error'];
            return FALSE;
        }
        // 値が全てある場合は時間不正チェック
        else {
            if ( !$this->vm_checktime($hour, 0, $this->params['time_kind'] - 1)
              || !$this->vm_checktime($minute, 0, 59)
              || !$this->vm_checktime($second, 0, 59) ) {

                $error = $this->params['time_error'];
                return FALSE;
            }
        }

        // 時間をUNIXタイムスタンプに変換
        $itime = mktime($hour, $minute, $second, 1, 1, 2000);

        if ( is_array($this->params['min']) ) {

            // 時間データ取得
            $minhour   = $this->params['min']['Hour'];
            $minminute = $this->params['min']['Minute'];
            $minsecond = $this->params['min']['Second'];

            //[ 秒が空？ ]
            if ( empty($minsecond) ) {
                $minsecond = $this->params['def_second'];
            }

            // 下限値をUNIXタイムスタンプに変換
            $min = mktime($minhour,
                          $minminute,
                          $minsecond,
                          1, 1, 2000);
            // 下限チェック
            if (!empty($this->params['min']) && $min > -1 && $itime < $min)
            {
                $error = $this->params['min_error'];
                return FALSE;
            }
        }

        if ( is_array($this->params['max']) ) {

            // 時間データ取得
            $maxhour   = $this->params['max']['Hour'];
            $maxminute = $this->params['max']['Minute'];
            $maxsecond = $this->params['max']['Second'];

            //[ 秒が空？ ]
            if ( empty($minsecond) ) {
                $maxsecond = $this->params['def_second'];
            }

            // 上限値をUNIXタイムスタンプに変換
            $max = mktime($maxhour,
                          $maxminute,
                          $maxsecond,
                          1, 1, 2000);
            // 上限チェック
            if (!empty($this->params['max']) && $max > -1 && $itime > $max)
            {
                $error = $this->params['max_error'];
                return FALSE;
            }
        }

        return TRUE;
    }

    //--------------------------------------------------------------------------
    //--    関数名： 数値チェック関数                                         --
    //--    Method： vm_checktime()                                           --
    //--------------------------------------------------------------------------
    //--  MODIFIED        YYYY-MM-DD REV    - NOTES                           --
    //--  NEOJAPAN        2005-10-28 1.00   - CREATE NEW                      --
    //--------------------------------------------------------------------------
    /**
     * 数値の妥当性をチェックする。
     *
     * @param    $time          int           数値
     * @param    $min           int           最小値
     * @param    $max           int           最大値
     * @return   true:正常、false:エラー
     */
    function vm_checktime ( $time, $min, $max )
    {

        if (!is_numeric($time))
        {
            return FALSE;
        }

        if ( $min > -1 && $time < $min)
        {
            return FALSE;
        }

        if ($max > -1 && $time > $max)
        {
            return FALSE;
        }

        return TRUE;

    }

}

?>