(function($) {
$.fn.extend({"categorySelecter" : function(userOptions) {
    var options = {
        url : $('#base_dir')[0].innerHTML + "index.php?module=dialog&action=ajax_getCatlist",
        idPrefix : "",
        labelAllCategories : "All Categories",
        dispAllCategories : "1"
    }

    /**
     * カテゴリーが選択された場合の処理(デフォルト)
     * 
     * @param   {event}     event   イベント
     * @type    {object}    idとnameを持つカテゴリーオブジェクトの配列
     */
    options.onSelect = function(event, categories){
        var id = "";
        var names = [];
        //[ カテゴリーのパスとカテゴリーIDを収集 ]
        $.each(categories, function(){
            id = this.id;
            names.push(this.name);
        });

        // inputにカテゴリーIDを設定
        $("input[name='cid']").val(id);
        // 要素にカテゴリーパスを設定
        var name = (names.length > 0) ? names.join(" > ") : options.labelAllCategories;
        $(".selected-category").text(name);
        $('.selected-category').linkwrapper();
        // inputにカテゴリーパスを設定
        $("input[name='cnm']").val(name);
        
        if ($(event.target).parents("#TB_window").length > 0){
            tb_remove();
        }
        // changeイベントを強制的に発生させる
        $("input[name='cid']").change();
    }

    $.extend(options, userOptions);

    /**
     * カテゴリーの選択を行う
     * 
     * @param   {jQuery}    item    選択要素(LI)
     * @type    {jQuery}    選択要素(LI)
     */
    var selectItem = function(item){
        item = $(item);
        item.siblings(".selected").removeClass("selected");
        item.addClass("selected");
        return item;
    }

    /**
     * イベントからカテゴリーの選択を行う
     * 
     * @param   {event}     event   イベント
     * @type    {jQuery}    選択要素(LI)
     */
    var selectItemByEvent = function(event){
        // クリックされたLI要素を取得する。
        var liElement = $(event.target);
        if (! liElement.is("li")){
            liElement = liElement.parents("li").slice(0, 1);
        }
        selectItem(liElement);

        // 下位カテゴリーを削除する。
        liElement.parent().nextAll().empty();
        return liElement;
    }

    /**
     * カテゴリーIDを取得する。
     * 
     * @param   {jQuery}    item    選択要素(LI)
     * @type    {string}    カテゴリーID
     */
    var getCategoryID = function(item){
        var id = $(item).attr("id");
        if (!id) {
            return;
        }
        var categoryID = id.match(/\d+/);
        if (!categoryID){
            return;
        }
    
        return categoryID+"";
    }

    /**
     * カテゴリー一覧を取得する。
     * 
     * @param   {element}   element     親要素
     * @param   {number}    colNo       列の番号
     * @param   {number}    catId       カテゴリーID
     * @param   {boolean}   hasChild    子カテゴリーがあればtrue
     */
    var loadCategories = function(element, colNo, catId, hasChild){
        /**
         * リストに追加する。
         * 
         * @param   {element}   ulElement   リストの要素
         * @param   {string}    id          カテゴリーID
         * @param   {string}    name        カテゴリー名
         * @param   {boolean}   isTail      子カテゴリーがなければtrue
         * @type    {element}   追加した要素
         */
        var addItemElement = function(ulElement, id, name, isTail){
            //[ A要素を追加 ]
            var aElement = document.createElement("A");
            aElement.href="javascript:void(0)";
            //[ カテゴリ名を追加 ]
            if (typeof(name) == "string"){
                var labelElement = document.createElement("SPAN");
                $(labelElement).addClass("label").text(name);
                aElement.appendChild(labelElement);
            }
            //[ 「>>」アイコンを追加 ]
            if (!isTail && colNo <= 1){
                var emphElement = document.createElement("SPAN");
                $(emphElement).addClass("emphasis").html("&raquo;");
                aElement.appendChild(emphElement);
            }
            //[ LI要素を追加 ]
            var liElement = document.createElement("LI");
            if (!!id){
                liElement.id = options.idPrefix+"category-" + id;
            }
            liElement.appendChild(aElement);
            // UL要素にLI要素を追加
            ulElement.appendChild(liElement);
            //[ A要素にイベントを設定 ]
            if (!!id && colNo <= 1){
                $(aElement).click(onOpenChildren);
            } else {
                $(aElement).click(onSelectCategory);
            }
        
            return liElement;
        }

        /**
         * カテゴリーが読み込まれた後に行う
         * 
         * @param   {object}    data    データ
         */
        var onLoadCategory = function(data){
            var ulElement = element.children().get(colNo);
            // カテゴリーの出力先列を初期化
            $(ulElement).empty();
            var liElement;
            if (options.dispAllCategories != '0' || colNo != '0') {
                liElement = addItemElement(ulElement, null, options.labelAllCategories, true);
                selectItem(liElement);
            }
            //[ カテゴリーをそれぞれ追加する。 ]
            $.each(data, function(){
                //var name = (typeof(this.name) == "string") ? unescape(this.name) : "?";
                //addItemElement(ulElement, this.id, name, this.tail)
                var name = (typeof(this.category_name) == "string") ? unescape(this.category_name) : "?";
                addItemElement(ulElement, this.category_id, name, this.tail)
            });
        }

        //[ カテゴリー未選択、又は子のあるカテゴリーが選択された場合
        if (typeof(hasChild) == "undefined" || !!hasChild){
            // サーバーからjsonを取得する。
            var params = {};
            if (typeof(catId) != "undefined"){
                params.id = catId;
            }
            $.getJSON(options.url, params, onLoadCategory);
        } else {
            // 子カテゴリがないものとする
            onLoadCategory({});
        }
    }

    /**
     * 子カテゴリーを開く場合に呼ばれる。
     * 
     * @param   {jQuery}    item    選択要素
     */
    var onOpenChildren = function(event){
        var liElement = selectItemByEvent(event);
        var ulElement = liElement.parent();
        var colNo = ulElement.prevAll().length;
        var id = getCategoryID(liElement);
        var hasChild = liElement.find(".emphasis").length > 0;

        // 子カテゴリーの読み込み
        loadCategories(ulElement.parent(), colNo+1, id, hasChild);
        event.preventDefault();
    }

    /**
     * カテゴリーが選択された場合に呼ばれる。
     * 
     * @param   {jQuery}    item    選択要素
     */
    var onSelectCategory = function(event, element){
        var liElement = selectItemByEvent(event);
        var categories = [];
        //[ 選択カテゴリーのパスを取得する。 ]
        var selectedElement = liElement.parents(".categorySelecter").find("li.selected[id]");
        selectedElement.each(function(index){
            var category = {};
            category.id = getCategoryID(this);
            category.name = $(this).find(".label").text(); 
            categories.push(category);
        });

        // 選択した場合の処理をする。
        options.onSelect(event, categories);
        event.preventDefault();
    }

    /**
     * 初期処理をする
     * 
     * @param   {jQuery}    element     選択要素
     */
    var init = function(element){
        element.addClass("categorySelecter");
        for(var i=0;i<3;i++){
            element.append("<ul></ul>");
        }
        element.find(":first-child").addClass("first-child");

        loadCategories(element, 0);
    }
    init(this);
}});
})(jQuery);
